@@ -8,8 +8,6 @@ import tomllib
 import json
 import os
 import sys
-from typing import cast
-from collections import defaultdict
 
 from .common import PyprError, get_logger, init_logger
 from .ipc import get_event_stream, notify_error, notify_fatal, notify_info
@@ -21,14 +19,16 @@ CONTROL = f'/tmp/hypr/{ os.environ["HYPRLAND_INSTANCE_SIGNATURE"] }/.pyprland.so
 OLD_CONFIG_FILE = "~/.config/hypr/pyprland.json"
 CONFIG_FILE = "~/.config/hypr/pyprland.toml"
 
+PYPR_DEMO = os.environ.get("PYPR_DEMO", False)
+
 
 class Pyprland:
     "Main app object"
     server: asyncio.Server
     event_reader: asyncio.StreamReader
     stopped = False
-    name = "builtin"
     config: None | dict[str, dict] = None
+    tasks: None | asyncio.TaskGroup = None
 
     def __init__(self):
         self.plugins: dict[str, Plugin] = {}
@@ -39,9 +39,6 @@ class Pyprland:
         "Initializes the main structures"
         await self.load_config()  # ensure sockets are connected first
 
-        for name in self.plugins:
-            self.queues[name] = asyncio.Queue()
-
     async def __open_config(self):
         """Loads config file as self.config"""
         if os.path.exists(OLD_CONFIG_FILE) and not os.path.exists(CONFIG_FILE):
@@ -76,13 +73,17 @@ class Pyprland:
         """Loads the plugins mentioned in the config.
         If init is `True`, call the `init()` method on each plugin.
         """
-        for name in cast(dict, self.config["pyprland"]["plugins"]):
+        init_pyprland = "pyprland" not in self.plugins
+        for name in ["pyprland"] + self.config["pyprland"]["plugins"]:
             if name not in self.plugins:
                 modname = name if "." in name else f"pyprland.plugins.{name}"
                 try:
                     plug = importlib.import_module(modname).Extension(name)
                     if init:
                         await plug.init()
+                        self.queues[name] = asyncio.Queue()
+                        if self.tasks:
+                            self.tasks.create_task(self._plugin_runner_loop(name))
                     self.plugins[name] = plug
                 except ModuleNotFoundError as e:
                     self.log.error("Unable to locate plugin called '%s'", name)
@@ -103,9 +104,12 @@ class Pyprland:
                     await notify_info(f"Error initializing plugin {name}: {e}")
                     self.log.error("Error initializing plugin %s:", name, exc_info=True)
                     raise PyprError() from e
+        if init_pyprland:
+            plug = self.plugins["pyprland"]
+            plug.set_commands(reload=self.load_config)
 
     async def load_config(self, init=True):
-        """Loads the configuration
+        """loads the configuration (new plugins will be added & config updated)
 
         if `init` is true, also initializes the plugins"""
 
@@ -115,12 +119,13 @@ class Pyprland:
 
     async def _run_plugin_handler(self, plugin, full_name, params):
         "Runs a single handler on a plugin"
-        self.log.debug("%s.%s%s", plugin.name, full_name, params)
+        color = 33 if full_name.startswith("run_") else 30
+        plugin.log.debug(f"\033[{color};1m%s%s\033[0m", full_name, params)
         try:
             await getattr(plugin, full_name)(*params)
         except AssertionError as e:
             self.log.error(
-                "Bug detected, please report on https://github.com/fdev31/pyprland/issues"
+                "This could be a bug in Pyprland, if you think so, report on https://github.com/fdev31/pyprland/issues"
             )
             self.log.exception(e)
             await notify_error(
@@ -134,7 +139,7 @@ class Pyprland:
     async def _callHandler(self, full_name, *params, notify=""):
         "Call an event handler with params"
         handled = False
-        for plugin in list(self.plugins.values()):
+        for plugin in self.plugins.values():
             if hasattr(plugin, full_name):
                 handled = True
                 await self.queues[plugin.name].put(
@@ -146,7 +151,6 @@ class Pyprland:
 
     async def read_events_loop(self):
         "Consumes the event loop and calls corresponding handlers"
-        last_cmd_args: dict[str, None | str] = defaultdict(lambda: None)
         while not self.stopped:
             try:
                 data = (await self.event_reader.readline()).decode()
@@ -157,13 +161,10 @@ class Pyprland:
                 self.log.critical("Reader starved")
                 return
             cmd, params = data.split(">>", 1)
-            last_args = last_cmd_args.get(cmd)
-            if params != last_args:
-                full_name = f"event_{cmd}"
+            full_name = f"event_{cmd}"
 
-                # self.log.debug("[%s] %s", cmd, params.strip())
-                await self._callHandler(full_name, params)
-            last_cmd_args[cmd] = params
+            # self.log.debug("[%s] %s", cmd, params.strip())
+            await self._callHandler(full_name, params.rstrip("\n"))
 
     async def read_command(self, reader, writer) -> None:
         "Receives a socket command"
@@ -186,9 +187,9 @@ class Pyprland:
             args = args[1:]
 
         full_name = f"run_{cmd}"
-        # Demos:
-        # run mako for notifications & uncomment this
-        # os.system(f"notify-send '{data}'")
+
+        if PYPR_DEMO:
+            os.system(f"notify-send -t 4000 '{data}'")
 
         if not await self._callHandler(full_name, *args, notify=cmd):
             self.log.warning("No such command: %s", cmd)
@@ -205,23 +206,30 @@ class Pyprland:
         "Runs tasks for a given plugin indefinitely"
         q = self.queues[name]
 
-        while True:
+        while not self.stopped:
             task = await q.get()
-            await task()
+            try:
+                await task()
+            except Exception as e:  # pylint: disable=W0718
+                self.log.error(
+                    "Unhandled error running plugin %s::%s: %s", name, task, e
+                )
+
+    async def plugins_runner(self):
+        "Runs plugins' task using the created `tasks` TaskGroup attribute"
+        async with asyncio.TaskGroup() as group:
+            self.tasks = group
+            for name in self.plugins:
+                group.create_task(self._plugin_runner_loop(name))
 
     async def run(self):
         "Runs the server and the event listener"
-        plugin_workers = [
-            asyncio.create_task(self._plugin_runner_loop(name)) for name in self.plugins
-        ]
         await asyncio.gather(
             asyncio.create_task(self.serve()),
             asyncio.create_task(self.read_events_loop()),
-            *plugin_workers,
+            asyncio.create_task(self.plugins_runner()),
         )
 
-    run_reload = load_config
-
 
 async def run_daemon():
     "Runs the server / daemon"
@@ -290,8 +298,7 @@ async def run_client():
 If the command is ommited, runs the daemon which will start every configured plugin.
 
 Available commands:
-
- reload               Reloads the config file (only supports adding or updating plugins)"""
+"""
         )
         for plug in manager.plugins.values():
             for name in dir(plug):
@@ -319,26 +326,56 @@ Available commands:
     await writer.wait_closed()
 
 
+def use_param(txt):
+    """Checks if parameter `txt` is in sys.argv
+    if found, removes it from sys.argv & returns the argument value
+    """
+    v = False
+    if txt in sys.argv:
+        i = sys.argv.index(txt)
+        v = sys.argv[i + 1]
+        del sys.argv[i : i + 2]
+    return v
+
+
 def main():
     "runs the command"
-    if "--debug" in sys.argv:
-        i = sys.argv.index("--debug")
-        init_logger(filename=sys.argv[i + 1], force_debug=True)
-        del sys.argv[i : i + 2]
+    debug_flag = use_param("--debug")
+    if debug_flag:
+        init_logger(filename=debug_flag, force_debug=True)
     else:
         init_logger()
     ipc_init()
     log = get_logger("startup")
-    try:
-        asyncio.run(run_daemon() if len(sys.argv) <= 1 else run_client())
-    except KeyboardInterrupt:
-        pass
-    except PyprError:
-        log.critical("Command failed.")
-    except json.decoder.JSONDecodeError as e:
-        log.critical("Invalid JSON syntax in the config file: %s", e.args[0])
-    except Exception:  # pylint: disable=W0718
-        log.critical("Unhandled exception:", exc_info=True)
+
+    config_override = use_param("--config")
+    if config_override:
+        global CONFIG_FILE
+        CONFIG_FILE = config_override
+
+    invoke_daemon = len(sys.argv) <= 1
+    if invoke_daemon and os.path.exists(CONTROL):
+        asyncio.run(notify_fatal("Trying to run pypr more than once ?"))
+        log.critical(
+            """%s exists,
+is pypr already running ?
+If that's not the case, delete this file and run again.""",
+            CONTROL,
+        )
+    else:
+        try:
+            asyncio.run(run_daemon() if invoke_daemon else run_client())
+        except KeyboardInterrupt:
+            pass
+        except PyprError:
+            log.critical("Command failed.")
+        except json.decoder.JSONDecodeError as e:
+            log.critical("Invalid JSON syntax in the config file: %s", e.args[0])
+        except Exception:  # pylint: disable=W0718
+            log.critical("Unhandled exception:", exc_info=True)
+        finally:
+            if invoke_daemon:
+                os.unlink(CONTROL)
 
 
 if __name__ == "__main__":
