@@ -2,6 +2,7 @@
 from typing import Any
 
 from ..common import get_logger
+from ..ipc import getCtrlObjects
 
 
 class Plugin:
@@ -11,6 +12,14 @@ class Plugin:
         "create a new plugin `name` and the matching logger"
         self.name = name
         self.log = get_logger(name)
+        ctrl = getCtrlObjects(self.log)
+        (
+            self.hyprctl,
+            self.hyprctlJSON,  # pylint: disable=invalid-name
+            self.notify,
+            self.notify_info,
+            self.notify_error,
+        ) = ctrl
         self.config: dict[str, Any] = {}
 
     async def init(self):
