@@ -8,21 +8,28 @@ from typing import Any, cast
 from functools import partial
 from collections import defaultdict
 
-from ..ipc import get_focused_monitor_props, hyprctl, hyprctlJSON
-from ..ipc import notify_error
+from ..ipc import notify_error, get_client_props, get_focused_monitor_props
 from .interface import Plugin
 
-DEFAULT_MARGIN = 60
+DEFAULT_MARGIN = 60  # in pixels
 AFTER_SHOW_INHIBITION = 0.2  # 200ms of ignorance after a show
 
 # Helper functions {{{
 
+invert_dimension = {"width": "height", "height": "width"}
+
+
+def get_space_identifier(obj):
+    "Returns a unique object for the workspace + monitor combination"
+    return (obj.workspace, obj.monitor)
+
 
 def convert_coords(logger, coords, monitor):
     """
     Converts a string like "X Y" to coordinates relative to monitor
     Supported formats for X, Y:
     - Percentage: "V%". V in [0; 100]
+    - Pixels: "Vpx". V should fit in your screen and not be zero
 
     Example:
     "10% 20%", monitor 800x600 => 80, 120
@@ -30,14 +37,20 @@ def convert_coords(logger, coords, monitor):
 
     assert coords, "coords must be non null"
 
-    def convert(size, dim):
+    def convert(size, dimension):
+        scale = float(monitor["scale"])
+        if monitor["transform"] in (1, 3):
+            dimension = invert_dimension[dimension]
         if size[-1] == "%":
             p = int(size[:-1])
             if p < 0 or p > 100:
                 raise ValueError(f"Percentage must be in range [0; 100], got {p}")
-            scale = float(monitor["scale"])
-            return int(monitor[dim] / scale * p / 100)
-        raise ValueError(f"Unsupported format for dimension {dim} size, got {size}")
+            return int(monitor[dimension] / scale * p / 100)
+        if size[-2:] == "px":
+            return int(size[:-2])
+        raise ValueError(
+            f"Unsupported format for dimension {dimension} size, got {size}"
+        )
 
     try:
         x_str, y_str = coords.split()
@@ -48,21 +61,6 @@ def convert_coords(logger, coords, monitor):
         raise e
 
 
-async def get_client_props(addr: str | None = None, pid: int | None = None):
-    "Returns client properties given its address"
-    assert addr or pid
-    if addr:
-        assert len(addr) > 2, "Client address is invalid"
-    if pid:
-        assert pid, "Client pid is invalid"
-    prop_name = "address" if addr else "pid"
-    prop_value = addr if addr else pid
-    for client in await hyprctlJSON("clients"):
-        assert isinstance(client, dict)
-        if client.get(prop_name) == prop_value:
-            return client
-
-
 # }}}
 
 
@@ -70,7 +68,7 @@ class Animations:  # {{{
     "Animation store"
 
     @staticmethod
-    async def fromtop(monitor, client, client_uid, margin):
+    def fromtop(monitor, client, client_uid, margin):
         "Slide from/to top"
         scale = float(monitor["scale"])
         mon_x = monitor["x"]
@@ -80,10 +78,10 @@ class Animations:  # {{{
         client_width = client["size"][0]
         margin_x = int((mon_width - client_width) / 2) + mon_x
 
-        await hyprctl(f"movewindowpixel exact {margin_x} {mon_y + margin},{client_uid}")
+        return f"movewindowpixel exact {margin_x} {mon_y + margin},{client_uid}"
 
     @staticmethod
-    async def frombottom(monitor, client, client_uid, margin):
+    def frombottom(monitor, client, client_uid, margin):
         "Slide from/to bottom"
         scale = float(monitor["scale"])
         mon_x = monitor["x"]
@@ -94,12 +92,10 @@ class Animations:  # {{{
         client_width = client["size"][0]
         client_height = client["size"][1]
         margin_x = int((mon_width - client_width) / 2) + mon_x
-        await hyprctl(
-            f"movewindowpixel exact {margin_x} {mon_y + mon_height - client_height - margin},{client_uid}"
-        )
+        return f"movewindowpixel exact {margin_x} {mon_y + mon_height - client_height - margin},{client_uid}"
 
     @staticmethod
-    async def fromleft(monitor, client, client_uid, margin):
+    def fromleft(monitor, client, client_uid, margin):
         "Slide from/to left"
         scale = float(monitor["scale"])
         mon_x = monitor["x"]
@@ -109,10 +105,10 @@ class Animations:  # {{{
         client_height = client["size"][1]
         margin_y = int((mon_height - client_height) / 2) + mon_y
 
-        await hyprctl(f"movewindowpixel exact {margin + mon_x} {margin_y},{client_uid}")
+        return f"movewindowpixel exact {margin + mon_x} {margin_y},{client_uid}"
 
     @staticmethod
-    async def fromright(monitor, client, client_uid, margin):
+    def fromright(monitor, client, client_uid, margin):
         "Slide from/to right"
         scale = float(monitor["scale"])
         mon_x = monitor["x"]
@@ -123,9 +119,7 @@ class Animations:  # {{{
         client_width = client["size"][0]
         client_height = client["size"][1]
         margin_y = int((mon_height - client_height) / 2) + mon_y
-        await hyprctl(
-            f"movewindowpixel exact {mon_width - client_width - margin + mon_x } {margin_y},{client_uid}"
-        )
+        return f"movewindowpixel exact {mon_width - client_width - margin + mon_x } {margin_y},{client_uid}"
 
 
 # }}}
@@ -139,31 +133,41 @@ class Scratch:  # {{{
         self.uid = uid
         self.pid = 0
         self.conf = opts
+        if not opts.get("process_tracking", True):
+            self.conf["lazy"] = True
+            self.conf["class_match"] = True
         self.visible = False
         self.client_info = {}
         self.should_hide = False
         self.initialized = False
         self.meta = {}
+        self.space_identifier = None
 
-    async def initialize(self):
+    async def initialize(self, ex):
         "Initialize the scratchpad"
         if self.initialized:
             return
         self.initialized = True
         await self.updateClientInfo()
-        await hyprctl(
+        await ex.hyprctl(
             f"movetoworkspacesilent special:scratch_{self.uid},address:{self.full_address}"
         )
 
-    def isAlive(self) -> bool:
+    async def isAlive(self) -> bool:
         "is the process running ?"
-        path = f"/proc/{self.pid}"
-        if os.path.exists(path):
-            with open(os.path.join(path, "status"), "r", encoding="utf-8") as f:
-                for line in f.readlines():
-                    if line.startswith("State"):
-                        state = line.split()[1]
-                        return state not in "ZX"  # not "Z (zombie)"or "X (dead)"
+        if self.conf.get("process_tracking", True):
+            path = f"/proc/{self.pid}"
+            if os.path.exists(path):
+                with open(os.path.join(path, "status"), "r", encoding="utf-8") as f:
+                    for line in f.readlines():
+                        if line.startswith("State"):
+                            state = line.split()[1]
+                            return state not in "ZX"  # not "Z (zombie)"or "X (dead)"
+        else:
+            if getattr(self, "bogus_pid", False):
+                return bool(await self.get_client_props(cls=self.conf["class"]))
+            return False
+
         return False
 
     def reset(self, pid: int) -> None:
@@ -181,12 +185,12 @@ class Scratch:  # {{{
     @property
     def full_address(self) -> str:
         "Returns the client address"
-        return self.client_info.get("address", "")
+        return cast(str, self.client_info.get("address", ""))
 
     async def updateClientInfo(self, client_info=None) -> None:
         "update the internal client info property, if not provided, refresh based on the current address"
         if client_info is None:
-            client_info = await get_client_props(addr=self.full_address)
+            client_info = await self.get_client_props(addr=self.full_address)
         if not isinstance(client_info, dict):
             self.log.error(
                 "client_info of %s must be a dict: %s", self.address, client_info
@@ -314,6 +318,24 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
 
     focused_window_tracking: dict[str, dict] = {}
 
+    workspace = ""  # Currently active workspace
+    monitor = ""  # CUrrently active monitor
+
+    def __init__(self, name):
+        super().__init__(name)
+        self.get_client_props = partial(get_client_props, logger=self.log)
+        Scratch.get_client_props = self.get_client_props
+        self.get_focused_monitor_props = partial(
+            get_focused_monitor_props, logger=self.log
+        )
+
+    async def init(self):
+        "Initializes the Scratchpad extension"
+        self.workspace = (await self.hyprctlJSON("activeworkspace"))["name"]
+        self.monitor = next(
+            mon for mon in (await self.hyprctlJSON("monitors")) if mon["focused"]
+        )
+
     async def exit(self) -> None:
         "exit hook"
 
@@ -321,10 +343,10 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             proc = self.procs[scratch.uid]
             proc.terminate()
             for _ in range(10):
-                if not scratch.isAlive():
+                if not await scratch.isAlive():
                     break
                 await asyncio.sleep(0.1)
-            if scratch.isAlive():
+            if await scratch.isAlive():
                 proc.kill()
             proc.wait()
 
@@ -355,46 +377,131 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             else:
                 self.log.error("Failure starting %s", name)
 
+        for scratch in list(self.scratches.getByState("configured")):
+            self.scratches.clearState(scratch, "configured")
+
+    async def _configure_windowrules(self, scratch):
+        "Setting up initial client window state (sets windowrules)"
+        configured = self.scratches.hasState(scratch, "configured")
+        if configured:
+            return
+        self.scratches.setState(scratch, "configured")
+        animation_type: str = scratch.conf.get("animation", "fromTop").lower()
+        defined_class: str = scratch.conf.get("class", "")
+        if animation_type and defined_class:
+            monitor = await get_focused_monitor_props(self.log)
+            width, height = convert_coords(
+                self.log, scratch.conf.get("size", "80% 80%"), monitor
+            )
+
+            position = scratch.conf.get("position")
+            if position:
+                margin_x, margin_y = convert_coords(self.log, position, monitor)
+            else:
+                margin_x = (monitor["width"] - width) // 2
+                margin_y = (monitor["height"] - height) // 2
+
+            t_pos = {
+                "fromtop": f"{margin_x} -200%",
+                "frombottom": f"{margin_x} 200%",
+                "fromright": f"200% {margin_y}",
+                "fromleft": f"-200% {margin_y}",
+            }[animation_type]
+
+            await self.hyprctl(
+                [
+                    f"windowrule workspace special:scratch_{scratch.uid} silent,^({defined_class})$",
+                    f"windowrule float,^({defined_class})$",
+                    f"windowrule move {t_pos},^({defined_class})$",
+                    f"windowrule size {width} {height},^({defined_class})$",
+                ],
+                "keyword",
+            )
+
+    async def __wait_for_client(self, item, use_proc=True) -> bool:
+        """Waits for a client to be up and running
+        if `class_match` is enabled, will use the class, else the process's PID will be used.
+        """
+        self.log.info("==> Wait for %s spawning", item.uid)
+        for loop_count in range(1, 8):
+            await asyncio.sleep(loop_count**2 / 10.0)
+            is_alive = await item.isAlive()
+
+            # skips the checks if the process isn't started (just wait)
+            if is_alive or not use_proc:
+                if item.conf.get("class_match"):
+                    info = await self.get_client_props(cls=item.conf.get("class"))
+                else:
+                    info = await self.get_client_props(pid=item.pid)
+                if info:
+                    await item.updateClientInfo(info)
+                    self.log.info(
+                        "=> %s client (proc:%s, addr:%s) detected on time",
+                        item.uid,
+                        item.pid,
+                        item.full_address,
+                    )
+                    self.scratches.register(item)
+                    self.scratches.clearState(item, "respawned")
+                    return True
+            if use_proc and not is_alive:
+                return False
+        return False
+
+    async def _start_scratch_classbased(self, item) -> bool:
+        "Ensure alive, PWA version"
+        uid = item.uid
+        started = getattr(item, "bogus_pid", False)
+        if not await item.isAlive():
+            started = False
+        if not started:
+            self.scratches.reset(item)
+            await self.start_scratch_command(uid)
+            r = await self.__wait_for_client(item, use_proc=False)
+            item.bogus_pid = True
+            return r
+        return True
+
+    async def _start_scratch(self, item):
+        "Ensure alive, standard version"
+        uid = item.uid
+        if uid in self.procs:
+            self.procs[uid].kill()
+        self.scratches.reset(item)
+        await self.start_scratch_command(uid)
+        self.log.info("starting %s", uid)
+        if not await self.__wait_for_client(item):
+            self.log.error("⚠ Failed spawning %s as proc %s", uid, item.pid)
+            if await item.isAlive():
+                error = "The command didn't open a window"
+            else:
+                self.procs[uid].communicate()
+                code = self.procs[uid].returncode
+                if code:
+                    error = f"The command failed with code {code}"
+                else:
+                    error = "The command terminated sucessfully, is it already running?"
+            self.log.error('"%s": %s', item.conf["command"], error)
+            await notify_error(error)
+            return False
+        return True
+
     async def ensure_alive(self, uid):
         """Ensure the scratchpad is started
         Returns true if started
         """
         item = self.scratches.get(name=uid)
+        await self._configure_windowrules(item)
 
-        if not item.isAlive():
-            self.log.info("%s is not running, restarting...", uid)
-            if uid in self.procs:
-                self.procs[uid].kill()
-            self.scratches.reset(item)
-            self.log.info("starting %s", uid)
-            await self.start_scratch_command(uid)
-            self.log.info("==> Wait for %s spawning", uid)
-            for loop_count in range(8):
-                if item.isAlive():
-                    if loop_count:
-                        await asyncio.sleep(loop_count**2 / 10.0)
-                    info = await get_client_props(pid=item.pid)
-                    if info:
-                        await item.updateClientInfo(info)
-                        self.log.info(
-                            "=> %s client (proc:%s, addr:%s) received on time",
-                            uid,
-                            item.pid,
-                            item.full_address,
-                        )
-                        self.scratches.register(item)
-                        self.scratches.clearState(item, "respawned")
-                        break
-            else:
-                self.log.error("⚠ Failed spawning %s as proc %s", uid, item.pid)
-                if item.isAlive():
-                    error = "The command didn't open a window"
-                else:
-                    error = "The command died"
-                self.log.error(error)
-                await notify_error(f"Failed to open {item.uid}: {error}")
-                return False
-        return True
+        if item.conf.get("process_tracking", True):
+            if not await item.isAlive():
+                self.log.info("%s is not running, starting...", uid)
+                if not await self._start_scratch(item):
+                    await notify_error(f'Failed to show scratch "{item.uid}"')
+                    return False
+            return True
+
+        return await self._start_scratch_classbased(item)
 
     async def start_scratch_command(self, name: str) -> None:
         "spawns a given scratchpad's process"
@@ -412,7 +519,9 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
         pid = proc.pid
         scratch.reset(pid)
         self.scratches.register(scratch, pid=pid)
-        self.log.info("scratch %s has pid %s", scratch.uid, pid)
+        self.log.info(
+            "scratch %s (%s) has pid %s", scratch.uid, scratch.conf.get("command"), pid
+        )
         if old_pid:
             self.scratches.clear(pid=old_pid)
 
@@ -420,7 +529,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
         """Update every scratchpads information if no `scratch` given,
         else update a specific scratchpad info"""
         pid = orig_scratch.pid if orig_scratch else None
-        for client in await hyprctlJSON("clients"):
+        for client in await self.hyprctlJSON("clients"):
             assert isinstance(client, dict)
             if pid and pid != client["pid"]:
                 continue
@@ -437,9 +546,22 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             self.log.info("Didn't update scratch info %s", self)
 
     # Events {{{
+    async def event_configreloaded(self, _nothing):
+        "Re-apply windowrules when hyprland is restarted"
+        for scratch in list(self.scratches.getByState("configured")):
+            self.scratches.clearState(scratch, "configured")
+            await self._configure_windowrules(scratch)
+
+    async def event_focusedmon(self, mon):
+        "focused monitor hook"
+        self.monitor = mon
+
+    async def event_workspace(self, workspace) -> None:
+        "workspace change hook"
+        self.workspace = workspace
+
     async def event_activewindowv2(self, addr) -> None:
         "active windows hook"
-        addr = addr.strip()
         for uid, scratch in self.scratches.items():
             if scratch.client_info and scratch.address != addr:
                 if (
@@ -457,15 +579,17 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
                         await self.run_hide(uid, autohide=True)
 
     async def _alternative_lookup(self):
-        "if class attribute is defined, use class matching and return True"
+        "if class_match attribute is defined, use class matching and return True"
         class_lookup_hack = [
-            s for s in self.scratches.getByState("respawned") if s.conf.get("class")
+            s
+            for s in self.scratches.getByState("respawned")
+            if s.conf.get("class_match")
         ]
         if not class_lookup_hack:
             return False
         self.log.debug("Lookup hack triggered")
         # hack to update the client info from the provided class
-        for client in await hyprctlJSON("clients"):
+        for client in await self.hyprctlJSON("clients"):
             assert isinstance(client, dict)
             for pending_scratch in class_lookup_hack:
                 if pending_scratch.conf["class"] == client["class"]:
@@ -488,7 +612,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             if item and item.should_hide:
                 await self.run_hide(item.uid, force=True)
         if item:
-            await item.initialize()
+            await item.initialize(self)
 
     # }}}
     # Commands {{{
@@ -500,7 +624,18 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             uids = [uid_or_uids.strip()]
 
         assert len(uids)
-        is_visible = self.scratches.get(uids[0]).visible
+        first_scratch = self.scratches.get(uids[0])
+        if not first_scratch:
+            self.log.warning("%s doesn't exist, can't toggle.", uids[0])
+            await notify_error(
+                f"Scratchpad '{uids[0]}' not found, check your configuration or the toggle parameter"
+            )
+            return
+
+        is_visible = (
+            first_scratch.visible
+            and first_scratch.space_identifier == get_space_identifier(self)
+        )
 
         tasks = []
 
@@ -510,7 +645,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
                 self.log.warning("%s is not configured", uid)
             else:
                 self.log.debug("%s is visible = %s", uid, item.visible)
-                if is_visible and item.isAlive():
+                if is_visible and await item.isAlive():
                     tasks.append(partial(self.run_hide, uid))
                 else:
                     tasks.append(partial(self.run_show, uid))
@@ -527,13 +662,13 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
 
         addr = "address:" + scratch.full_address
         if animation_type == "fromtop":
-            await hyprctl(f"movewindowpixel 0 -{offset},{addr}")
+            await self.hyprctl(f"movewindowpixel 0 -{offset},{addr}")
         elif animation_type == "frombottom":
-            await hyprctl(f"movewindowpixel 0 {offset},{addr}")
+            await self.hyprctl(f"movewindowpixel 0 {offset},{addr}")
         elif animation_type == "fromleft":
-            await hyprctl(f"movewindowpixel -{offset} 0,{addr}")
+            await self.hyprctl(f"movewindowpixel -{offset} 0,{addr}")
         elif animation_type == "fromright":
-            await hyprctl(f"movewindowpixel {offset} 0,{addr}")
+            await self.hyprctl(f"movewindowpixel {offset} 0,{addr}")
 
         if self.scratches.hasState(scratch, "transition"):
             return  # abort sequence
@@ -541,11 +676,17 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
 
     async def run_show(self, uid) -> None:
         """<name> shows scratchpad "name" """
-        uid = uid.strip()
         item = self.scratches.get(uid)
 
+        if not item:
+            self.log.warning("%s doesn't exist, can't hide.", uid)
+            await notify_error(
+                f"Scratchpad '{uid}' not found, check your configuration or the show parameter"
+            )
+            return
+
         self.focused_window_tracking[uid] = cast(
-            dict[str, Any], await hyprctlJSON("activewindow")
+            dict[str, Any], await self.hyprctlJSON("activewindow")
         )
 
         if not item:
@@ -562,14 +703,16 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             excluded = [
                 scratch.uid for scratch in self.scratches.values() if scratch.uid != uid
             ]
-        for tbh_scratch in excluded:
-            self.log.info("hidding ")
-            await self.run_hide(tbh_scratch, autohide=True)
+        for e_uid in excluded:
+            scratch = self.scratches.get(e_uid)
+            if scratch.visible:
+                await self.run_hide(e_uid, autohide=True)
         await item.updateClientInfo()
-        await item.initialize()
+        await item.initialize(self)
 
         item.visible = True
-        monitor = await get_focused_monitor_props()
+        item.space_identifier = get_space_identifier(self)
+        monitor = await get_focused_monitor_props(self.log)
         assert monitor
         assert item.full_address, "No address !"
 
@@ -578,18 +721,26 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
 
         self.scratches.setState(item, "transition")
         # Start the transition
-        await hyprctl(f"moveworkspacetomonitor special:scratch_{uid} {monitor['name']}")
-        await hyprctl(f"movetoworkspacesilent {wrkspc},address:{item.full_address}")
+        await self.hyprctl(
+            [
+                f"moveworkspacetomonitor special:scratch_{uid} {monitor['name']}",
+                f"movetoworkspacesilent {wrkspc},address:{item.full_address}",
+                f"alterzorder top,address:{item.full_address}",
+            ]
+        )
+        await self._fix_size_and_position(item, monitor)
         if animation_type:
             margin = item.conf.get("margin", DEFAULT_MARGIN)
             fn = getattr(Animations, animation_type)
-            await fn(monitor, item.client_info, "address:" + item.full_address, margin)
+            command = fn(
+                monitor, item.client_info, "address:" + item.full_address, margin
+            )
+            await self.hyprctl(command)
 
-        await hyprctl(f"focuswindow address:{item.full_address}")
+        await self.hyprctl(f"focuswindow address:{item.full_address}")
         await asyncio.sleep(0.2)  # ensure some time for events to propagate
         # Transition ended
         self.scratches.clearState(item, "transition")
-        await self._fix_size_and_position(item, monitor)
         item.meta["last_shown"] = time.time()
 
     async def _fix_size_and_position(self, item, monitor):
@@ -600,25 +751,36 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
         if position:
             x_pos, y_pos = convert_coords(self.log, position, monitor)
             x_pos_abs, y_pos_abs = x_pos + monitor["x"], y_pos + monitor["y"]
-            await hyprctl(
+            await self.hyprctl(
                 f"movewindowpixel exact {x_pos_abs} {y_pos_abs},address:{item.full_address}"
             )
         if size:
-            x_size, y_size = convert_coords(self.log, size, monitor)
-            await hyprctl(
-                f"resizewindowpixel exact {x_size} {y_size},address:{item.full_address}"
+            width, height = convert_coords(self.log, size, monitor)
+            max_size = item.conf.get("max_size")
+            if max_size:
+                max_width, max_height = convert_coords(self.log, max_size, monitor)
+                width = min(max_width, width)
+                height = min(max_height, height)
+            await self.hyprctl(
+                f"resizewindowpixel exact {width} {height},address:{item.full_address}"
             )
+        if size or position:
+            await item.updateClientInfo()
 
     async def run_hide(self, uid: str, force=False, autohide=False) -> None:
         """<name> hides scratchpad "name"
         if `autohide` is True, skips focus tracking
         `force` ignores the visibility check"""
-        uid = uid.strip()
         scratch = self.scratches.get(uid)
+
         if not scratch:
+            await notify_error(
+                f"Scratchpad '{uid}' not found, check your configuration or the hide parameter"
+            )
             self.log.warning("%s is not configured", uid)
             return
-        if not scratch.visible and not force:
+        if not scratch.visible and not force and not autohide:
+            await notify_error(f"Scratchpad '{uid}' is not visible, will not hide.")
             self.log.warning("%s is already hidden", uid)
             return
         scratch.visible = False
@@ -628,7 +790,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             await self._anim_hide(animation_type, scratch)
 
         if not self.scratches.hasState(scratch, "transition"):
-            await hyprctl(
+            await self.hyprctl(
                 f"movetoworkspacesilent special:scratch_{uid},address:{scratch.full_address}"
             )
 
@@ -636,7 +798,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             animation_type and uid in self.focused_window_tracking
         ):  # focus got lost when animating
             if not autohide and "address" in self.focused_window_tracking[uid]:
-                await hyprctl(
+                await self.hyprctl(
                     f"focuswindow address:{self.focused_window_tracking[uid]['address']}"
                 )
                 del self.focused_window_tracking[uid]
